/****
 * sstest.c                     Andrew Huang                Xtreme Ideas  
 *
 * 11/29/97                                                  Mt View, CA
 ****/

/*** defines ***/
#undef  LINUX 0    /* activate linux compatibility stuff */
#define DOS 1

#define SSTEP 0
#define RHPDEBUG 0
#define WAIT_ON 1 

#define WAIT  50

/* strobe is CR_STB */
/* frame is CR_SEL */
#define NRZ_CLK  (CR_STB)
#define FRAME    (CR_SEL)

/* command defines */
#define LED_CMD    0
#define SS_AL_CMD  1
#define SS_AH_CMD  2
#define SS_WDL_CMD 3
#define SS_WDH_CMD 4
#define SS_RDL_CMD 5
#define SS_RDH_CMD 6
#define SS_RD_CMD  14
#define SS_WR_CMD  15

#define WAIT_LEN 100

/*** includes ***/
#include "pport.h"

/*** typedefs ***/

/*** globals ***/

/*** func protos ***/
void init_ports();
void do_in_cmd( unsigned char cmd, unsigned char *data );
void do_out_cmd( char cmd, char data );
void set_adr( unsigned long data );
void set_wd( unsigned long data );
void set_led( char led );
void commit_wr();
void load_data();
void get_data( unsigned long *data );

void init_ports() {
  /* init_ports */
  /* locals */

  /* body */

#ifdef LINUX
  ioperm((unsigned long) LP1_BASE, (unsigned long) 0x3, 0x2 );
#endif

} /* init_ports */

void do_in_cmd( unsigned char cmd, unsigned char *data ) 
{ /* do_in_cmd */
  /* locals */
  char n;
  char state, out;
  char in;
  int  i;

  /* body */
  n = 0;  /* change later to change node ID */

  state = inb( LP1_CR );

  outb( (char) n & 0xFF, LP1_DR );  /* node id */
  out = state & ~FRAME & ~NRZ_CLK;
  outb( (char) out & 0xFF, LP1_CR );  /* frame and strobe high */
  for( i = 0; i < WAIT_LEN; i++ )
   ;
  
  outb( (char) cmd, LP1_DR );  
  out = state | (FRAME | NRZ_CLK);
  outb( (char) out & 0xFF, LP1_CR );  /* frame, strobe low */
  outb( (inb( LP1_CR ) | CR_BID) & 0xFF, LP1_CR ); /* turn on bidir */
  for( i = 0; i < WAIT_LEN; i++ )
   ;
  in = inb( LP1_DR );  
  *data = in;

  out = (state & ~NRZ_CLK) | FRAME;
  outb( (char) out & 0xFF, LP1_CR );  
  for( i = 0; i < WAIT_LEN; i++ )
   ;
  outb( (inb( LP1_CR ) & ~CR_BID) & 0xFF, LP1_CR ); /* turn off bidir */
    
  out = (state | FRAME | NRZ_CLK);
  outb( (char) out & 0xFF, LP1_CR ); 
  for( i = 0; i < WAIT_LEN; i++ )
   ;
  /* set frame and nrz clock to zero */

} /* do_in_cmd */
  
void do_out_cmd( char cmd, char data ) 
{ /* do_out_cmd */
  /* locals */
  char n;
  char out, state;
  int  i;

  /* body */
  n = 0;  /* change later to change node ID */

  state = inb( LP1_CR );

  outb( (char) n & 0xFF, LP1_DR );  /* node id */
  out = state & ~FRAME & ~NRZ_CLK;
  outb( (char) out & 0xFF, LP1_CR );  /* frame and strobe high */
  for( i = 0; i < WAIT_LEN; i++ )
   ;

  outb( (char) cmd, LP1_DR );  
  out = state | (FRAME | NRZ_CLK);
  outb( (char) out & 0xFF, LP1_CR );  /* frame, strobe low */
  for( i = 0; i < WAIT_LEN; i++ )
   ;
    
  outb( (char) data & 0xFF, LP1_DR );
  out = (state & ~NRZ_CLK) | FRAME;
  outb( (char) out & 0xFF, LP1_CR );  
  for( i = 0; i < WAIT_LEN; i++ )
   ;
    
  out = (state | FRAME | NRZ_CLK);
  outb( (char) out & 0xFF, LP1_CR ); 
  for( i = 0; i < WAIT_LEN; i++ )
   ;
  /* set frame and nrz clock to zero */

} /* do_out_cmd */

void set_adr( unsigned long adr ) {
  /* set_adr */
  /* locals */
  char low, high;

  /* body */
  low = adr & 0xFF;
  high = (adr >> 8) & 0xFF;

  do_out_cmd( SS_AL_CMD, low );
  do_out_cmd( SS_AH_CMD, high );
  
} /* set_adr */

void set_wd( unsigned long data ) {
  /* set_wd */
  /* locals */
  char low, high;
  
  /* body */
  low = data & 0xFF;
  high = (data >> 8) & 0xFF;

  do_out_cmd( SS_WDL_CMD, low ); 
  do_out_cmd( SS_WDH_CMD, high );

} /* set_wd */

void set_led( char led ) {
  do_out_cmd( LED_CMD, led );
} /* set_led */

void commit_wr() {
  do_out_cmd( SS_WR_CMD, 0 );
} /* commit_wr */

void load_data() {
  do_out_cmd( SS_RD_CMD, 0 );
} /* load_data */

void get_data( unsigned long *data ) {
  /* get_data */
  /* locals */
  unsigned char high, low;
  unsigned long temp;

  /* body */
  do_in_cmd( SS_RDL_CMD, &low );
  do_in_cmd( SS_RDH_CMD, &high );

  temp = 0L;
  temp = 0xFF & (temp | (0xFF & high));
  temp <<= 8;
  temp = 0xFFFF & (temp | (0xFF & low ));

/*  printf("temp: %04X ", temp ); */
  *data = temp;
  
} /* get_data */

#define TEST_SIZE 1024
#define MEM_SIZE 65536

void main( int argc, char *argv[] ) {
  /* main */
  /* locals */
  long   i;
  char   c;
  unsigned long mem[ TEST_SIZE ];
  unsigned long data;
  long   errs = 0;
  long   j, k;
  long   offset = 0;

  /* body */
  /***** 
   * init any machine-specific stuph 
   *****/
  init_ports();
  
  outb( (char) (inb( LP1_CR ) | FRAME | NRZ_CLK) & 0xFF, LP1_CR ); 
       /* set frame and nrz clock to zero */

for( j = 0; ; j++ ) {
  for( i = 0; i < TEST_SIZE; i++ ) {
    mem[i] = (rand() * rand()) & 0xFFFF;
/*    mem[i] = 0xFF; */
    set_led( mem[i] );
  } /* for */

/*  for( i = 0; ; i++ ) {
    set_led( (char) ~i & 0xFF );
    set_adr(i + offset);
    set_wd( i + 1 );
    commit_wr();
  } */

  for( i = 0; i < TEST_SIZE; i++ ) {
    set_led( (char) i );
    set_adr( i + offset );
    set_wd( (unsigned long) mem[i] );
    commit_wr();
  } /* for */

  /* errs = 0; */ /* don't clear errs to zero; cumulative test */
  for( i = 0; i < TEST_SIZE; i++ ) {
    set_led( (char) ~mem[i] );
    set_adr( i + offset );
    load_data();
    get_data( &data );
    if( (data & 0xFFFF) AINT (mem[i] & 0xFFFF) ) {
      errs++;
      printf( "Error at adr %03lX offset %05lX: got %04lX should be %04lX\n", i, offset, data & 0xFFFF, mem[i] & 0xFFFF );
    } /* if */
  } /* for */

  printf( "Memory test finished; %d errors, %d tested, %ld offset.\n", (int) errs, TEST_SIZE, offset );
  for( k = 0; k < 2000000; k++ )
   ;

  offset += TEST_SIZE;
  offset = offset % MEM_SIZE;
}

} /* main */
